/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package de.abien.eis.api.persistent;

import java.io.Serializable;
import java.util.Date;
import javax.persistence.Column;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.MappedSuperclass;
import javax.persistence.PrePersist;
import javax.persistence.PreUpdate;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Version;
import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;

/**
 *
 * This class is a sample for the @MappedSuperclass annotation. It is used for
 * all entities whose primary key is a {@code Long}.
 *
 * $Id: Persistent.java 7281 2009-10-15 10:43:20Z jhelwich $
 * @author <a href="mailto:norman.schoeneich@edict.de">Norman Sch&ouml;neich</a>
 *
 * @since 0.8
 */
@MappedSuperclass
@XmlAccessorType(XmlAccessType.FIELD)
public abstract class Persistent implements Serializable {

    /**
     * Dummy user ID to be saved in created_by / modified_by columns as long as
     * no real values are supplied.
     * TODO remove when a user ID is supplied
     */
    private static final Long DUMMY_USER_ID = 1l;
    private static final long serialVersionUID = 1L;
    @Id
    @GeneratedValue(strategy = GenerationType.SEQUENCE)
    private Long id;

    @Temporal(TemporalType.TIMESTAMP)
    @Column(nullable = false)
    protected Date created;

    @Column(name = "CREATED_BY", nullable = false)
    protected Long createdBy;

    @Temporal(TemporalType.TIMESTAMP)
    private Date modified;

    @Column(name = "MODIFIED_BY")
    private Long modifiedBy;

    @Version
    private int version;

    /** Returns the primary key of this entity. The id is auto-generated.
     *
     * @return the primary key of this entity.
     */
    public Long getId() {
        return id;
    }

    protected void setId(Long id) {
        this.id = id;
    }

    public Date getCreated() {
        return (created != null) ? new Date(created.getTime()) : null;
    }

    public Long getCreatedBy() {
        return createdBy;
    }

    public Date getModified() {
        return (modified != null) ? new Date(modified.getTime()) : null;
    }

    public Long getModifiedBy() {
        return modifiedBy;
    }

    public int getVersion() {
        return version;
    }

    public void setVersion(int version) {
        this.version = version;
    }

    @PrePersist
    public void prePersist() {
        // method needs to be public, otherwise PrePersist is ignored
        this.created = new Date();
        this.createdBy = DUMMY_USER_ID;
    }

    @PreUpdate
    public void preUpdate() {
        modified = new Date();
        modifiedBy = DUMMY_USER_ID;
    }

    /** Returns the hashcode of this entity, which is simply the hashcode of the
     * id, if it already persisted. If not {@code 0} is returned.
     *
     * @return the hashcode of this entity.
     */
    @Override
    public int hashCode() {
        int hash = 0;
        hash += (id != null ? id.hashCode() : 0);
        return hash;
    }

    /** Indicates whether the given object is "equal to" this one. An object
     * is considered to be equal if it is of the same class and the ids are the
     * same. If the id is {@code null} object equality is checked.
     *
     * @param object the object to check for equality.

     * @return {@code true} if the objects are equal, {@code false} otherwise.
     */
    @Override
    @SuppressWarnings("unchecked")
    public boolean equals(Object object) {
        if (object == null) {
            return false;
        }
        // this is a trick to let this equals work for subclasses
        if (!(getClass().isInstance(object))) {
            return false;
        }
        if (this.id == null) {
            // if we have no id, we check object identity
            return super.equals(object);
        }
        Persistent other = (Persistent) object;
        return this.id.equals(other.id);
    }

    /** Returns a simple string description. Actually the classname and the id.
     *
     * @return a simple string description.
     */
    @Override
    public String toString() {
        return getClass().getName() + "[id=" + id + "]" + "[version=" + version + "]";
    }
}
