package de.abien.eis.api.playermgmt;

/**
 *
 * @author fcap
 */
public interface PlayermgmtService {

    /**
     * Logs in a player with the given username and password. The State has to be
     * active or temppassword
     * @param username the username
     * @param password the password
     * @return the player or {@code null} if the given combination does not reference a player
     * @throws if player unknown or in wrong state
     */
    Player login(String username, String password, String sessionId);

    /**
     * Logs out the given player
     * @param player the Player
     */
    void logout(Player player);

    /**
     * Returns the registration Question
     * @param username
     * @return the registration Question
     * @throws if player unknown or in wrong state
     */
    String forgotPasswordGetQuestion(String usernameOrEmail);

    /**
     * Returns and persists a new generated password
     * @param username
     * @param registrationAnswer the answer of the registrationQuestion
     * @return new generated password
     * @throws if user unknown or in wrong state
     */
    String forgotPassword(String usernameOrEmail, String registrationAnswer);

    /**
     * Persists a new player
     * @param player the player
     * @return email confirm value
     * @throws if email or username already in database
     */
    String createPlayer(Player player);

    /**
     * Returns email confirm value (if the user lost the email)
     * @param username
     * @return email confirm value
     * @throws if the Player was not found or not in State registered
     */
    String forgotEmailConfirmValue(String usernameOrEmail);

    /**
     * Confirm players email and activated player account
     * @param username username of the player
     * @param emailConfirmValue sent by email to player
     * @return Player found with status activated
     * @throws if player unknown or already confirmed/in wrong state
     */
    Player confirmEmail(String username, String emailConfirmValue);

    /**
     * Returns if the username is available and cleansup lazy
     * @param username the username
     * @return if the username is available
     */
    boolean isUsernameAvailable(String username);

    /** 
     * Returns if the email is already registered and cleansup lazy
     * @param email the email
     * @return if the email is already registered
     */
    boolean isEmailNotInUse(String email);

    /**
     * Returns if the email is already registered and cleansup lazy
     * Unconfirmed email addresses of the given user will be ignored.
     *
     *
     * @param player the player
     * @param email the email
     * @return if the email is already registered
     */
    boolean isEmailNotInUse(Player player, String email);

    /**
     * Disable a player and
     * @param player the player
     * @param forceCancelation ignore validations like depot balance = 0
     * @throws when canceling of account impossible/player unknown
     */
    void cancelPlayer(Player player, boolean forceCancelation);

    /**
     * Updates the given detached player.
     * @param player the player.
     */
    Player updatePlayer(Player player);

    /**
     * Returns the player with the given SessionID
     * @param sessionId
     * @return player with given SessionID
     * @exception when none or more than one player found
     */
    Player getPlayerForSessionId(String sessionId);

    /**
     * Returns the player with the given eMail address or username
     * @param usernameOrEmail may be eMail address or username
     * @return player found for the given parameter
     * @exception when none or more than one player found
     */
    Player getPlayerForUsernameOrEmail(String usernameOrEmail);

    /**
     * Deletes the player from the DB.
     * WARNING: PLAYER WILL BE IRREVOCABLY DELETED!
     *
     */
    void deletePlayer(Player player);

    Long countAllPlayers();

}